/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.internal.ReportNodeManager");

dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.require("com.ibm.rational.report.web.core.app.view.navigation.ReportNodeParser");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");

var CoreServiceabilityMessages = dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var ReportClient 			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var session					= com.ibm.team.repository.web.client.session;
var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

/**
 * Manages the report folders and nodes in the Reports drawer.
 */
com.ibm.rational.report.web.core.app.internal.ReportNodeManager = {
	
	initialized: false,
	
	// create a dummy parser first to avoid null pointer exception
	_nodeParser: new com.ibm.rational.report.web.core.app.view.navigation.ReportNodeParser(null),	
	_doneCallbacks: [],
	_errorCallbacks: [],
	_isProcessing: false,
	_msgFormatter: new MessageFormatter(null),
	
	getNodeParser: function() {
		return this._nodeParser;
	},
	
	refresh: function(doneCallback, errorCallback) {
		this._doneCallbacks.push(doneCallback);
		this._errorCallbacks.push(errorCallback);
		
		if (this._isProcessing) {
			return;
		}
		
		this._isProcessing = true;
		
		var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		var owner =  session.getAuthenticatedContributor();		
		var params = {projectAreaId: projectArea.itemId, ownerId: owner.itemId};
		
		var srh = new ServiceResponseHandler(this, "_parseData", "_remoteError");				
		ReportClient.getReportTreeNodes(srh, params);	
	},
	
	_parseData: function(data) {
		this._nodeParser.refresh(data);
		
		while (this._doneCallbacks.length > 0) {
			var f = this._doneCallbacks.pop();
			if (f != null) f();
		}
		
		this._doneCallbacks = [];
		this._errorCallbacks = [];
		
		this.initialized = true;
		this._isProcessing = false;
	},
	
	_remoteError: function(error) {
		if (this._errorCallbacks.length > 0) {
			while (this._errorCallbacks.length > 0) {
				var f = this._errorCallbacks.pop();
				if (f != null) f(error);
			}
		} else {
			var msg = this._msgFormatter.formatErrorMessage(CoreServiceabilityMessages.FAILED_TO_RETRIEVE_REPORT_FOLDERS, error);
			this._displayErrorMessage(msg);
		}
		
		this._doneCallbacks = [];
		this._errorCallbacks = [];
		
		this.initialized = true;
		this._isProcessing = false;
	},
	
	_displayErrorMessage: function(errorMsg) {
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);
	}
};

})();
